﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AdventOfCode2023.Day2
{
    public class Day2_1 : DayBase
    {
        string input = string.Empty;

        Regex gameIdRegex = new Regex(@"Game (?<ID>\d+):");

        public Day2_1()
        {
            input = GetInput("Day2.txt");
        }

        public Day2_1(string testInput)
        {
            input = testInput;
        }

        public override string Execute()
        {
            var inputLines = input.Split("\r\n");
            List<Game> games = inputLines.Select(LineToGame).ToList();

            foreach (var game in games)
            {
                var maxima = game.GetMaxima();
                if (maxima.red > 12 || maxima.green > 13 || maxima.blue > 14)
                {
                    game.Possible = false;
                }
            }

            return games.Select(x => x.Possible ? x.GetId() : 0).Sum().ToString();
        }

        private Game LineToGame(string line)
        {
            if(!gameIdRegex.IsMatch(line))
            {
                throw new Exception($"Line: {line} does not contain game id!");
            }

            string gameId = gameIdRegex.Match(line).Groups["ID"].Value;
            string setLine = gameIdRegex.Replace(line, "");
            List<string> set = setLine.Split(';').ToList();

            return new Game(gameId, set);
        }
    }

    internal class Game
    {
        string id;
        List<string> sets = new List<string>();
        Regex amountColorRegex = new Regex(@"(?<AMOUNT>\d+) (?<COLOR>.+?)(,|$)");

        internal bool Possible = true;
        internal int Power = 0;

        internal Game(string Id, List<string> Sets)
        {
            id = Id;
            sets = Sets;
        }

        internal int GetId()
        {
            return int.Parse(id);
        }

        internal (int red, int green, int blue) GetMaxima()
        {
            List<int> blue = new List<int>();
            List<int> green = new List<int>();
            List<int> red = new List<int>();

            foreach (var set in sets)
            {
                var matches = amountColorRegex.Matches(set);
                foreach (Match match in matches)
                {
                    string color = match.Groups["COLOR"].Value;
                    int amount = int.Parse(match.Groups["AMOUNT"].Value);

                    switch (color)
                    {
                        case "red":
                            red.Add(amount);
                            break;
                        case "green":
                            green.Add(amount);
                            break;
                        case "blue":
                            blue.Add(amount);
                            break;
                        default:
                            throw new Exception("something wrong!");
                    }
                }
            }

            return (red.Max(), green.Max(), blue.Max());
        }
    }
}
