﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AdventOfCode2023.Day4
{
    public class Day4_2 : DayBase
    {
        string input = string.Empty;

        Regex rxWinningNumbers = new Regex(@"^.+?: (?<numbers>[ \d]*) $");
        Regex rxDrawnNumbers = new Regex(@"^ (?<numbers>[ \d]*)$");

        public Day4_2()
        {
            input = GetInput("Day4.txt");
        }

        public Day4_2(string testInput)
        {
            input = testInput;
        }

        public override string Execute()
        {
            var cardLines = input.Split("\r\n");
            List<Card> cards = cardLines.Select(line => new Card(1, line)).ToList();

            int index = 0;

            foreach (var card in cards)
            {
                var cardParts = card.cardValue.Split('|');

                string winningSequence = rxWinningNumbers.Match(cardParts[0]).Groups["numbers"].Value;
                string drawnSequence = rxDrawnNumbers.Match(cardParts[1]).Groups["numbers"].Value;

                List<int> winningNumbers = new List<int>();
                List<int> drawnNumbers = new List<int>();

                for (int i = 0; i < winningSequence.Length; i += 3)
                {
                    winningNumbers.Add(int.Parse(winningSequence.Substring(i, 2).Trim()));
                }
                for (int i = 0; i < drawnSequence.Length; i += 3)
                {
                    drawnNumbers.Add(int.Parse(drawnSequence.Substring(i, 2).Trim()));
                }

                var intersections = drawnNumbers.Intersect(winningNumbers).ToList();

                int points = intersections.Count;
                for (int i = 1; i <= points; i++)
                {
                    cards[index + i].amount += (1 * card.amount);
                }

                index++;
            }

            return cards.Sum(x => x.amount).ToString();
        }
    }

    internal class Card
    {
        public int amount;
        public string cardValue;

        internal Card(int amount, string cardValue)
        {
            this.amount = amount;
            this.cardValue = cardValue;
        }
    }
}
