﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AdventOfCode2023.Day5
{
    public class Day5_1 : DayBase
    {
        string input = string.Empty;

        Regex rxSeeds = new Regex(@"(?<=seeds:[\s\d]+)\d+");
        Regex rxSeedToSoil = new Regex(@"(?<=seed-to-soil map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxSoilToFertilizer = new Regex(@"(?<=soil-to-fertilizer map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxFertilizerToWater = new Regex(@"(?<=fertilizer-to-water map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxWaterToLight = new Regex(@"(?<=water-to-light map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxLightToTemperature = new Regex(@"(?<=light-to-temperature map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxTemperatureToHumidity = new Regex(@"(?<=temperature-to-humidity map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxHumidityToLocation = new Regex(@"(?<=humidity-to-location map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");

        public Day5_1()
        {
            input = GetInput("Day5.txt");
        }

        public Day5_1(string testInput)
        {
            input = testInput;
        }

        public override string Execute()
        {
            List<ulong> seeds = rxSeeds.Matches(input).Select(x => ulong.Parse(x.Value)).ToList();
            List<Map> seedToSoilMaps = rxSeedToSoil.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> soilToFertilizer = rxSoilToFertilizer.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> fertilizerToWater = rxFertilizerToWater.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> waterToLight = rxWaterToLight.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> lightToTemperature = rxLightToTemperature.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> temperatureToHumidity = rxTemperatureToHumidity.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> humidityToLocation = rxHumidityToLocation.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();

            List<ulong> locations = seeds.Select(x => seedToSoilMaps.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).Select(x => soilToFertilizer.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).Select(x => fertilizerToWater.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).Select(x => waterToLight.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).Select(x => lightToTemperature.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).Select(x => temperatureToHumidity.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).Select(x => humidityToLocation.Find(y => y.IsInSourceRange(x))?.Convert(x) ?? x).ToList();

            return locations.Min().ToString();
        }
    }

    internal class Map
    {
        private ulong _destinationRangeStart;
        private ulong _destinationRangeEnd;
        private ulong _sourceRangeStart;
        private ulong _rangeLength;
        private ulong _sourceRangeEnd;

        public ulong DestinationRangeStart { get => _destinationRangeStart; }
        public ulong DestinationRangeEnd { get => _destinationRangeEnd; }

        public Map(ulong destinationRangeStart, ulong sourceRangeStart, ulong rangeLength)
        {
            _destinationRangeStart = destinationRangeStart;
            _sourceRangeStart = sourceRangeStart;
            _rangeLength = rangeLength;
            _sourceRangeEnd = _sourceRangeStart + (_rangeLength - 1);
            _destinationRangeEnd = destinationRangeStart + (_rangeLength - 1);
        }

        public bool IsInSourceRange(ulong input) => input >= _sourceRangeStart && input <= _sourceRangeEnd;

        public bool IsInDestinationRange(ulong input) => input >= _destinationRangeStart && input <= _destinationRangeEnd;

        public ulong Convert(ulong source)
        {
            return source + (_destinationRangeStart - _sourceRangeStart);
        }

        public (ulong rangeStart, ulong rangeEnd) GetSourceRange() => (_sourceRangeStart, _sourceRangeEnd);
    }
}